def for_test():
    pass


class Some:
    def __init__(self):
        pass

    def for_test(self):
        pass


class Another:
    def __init__(self):
        pass

    def for_test(self):
        pass
